/*
*	OEM_flash.C
*
*   OEM data flash related functions.
*
*	Author: Hugo Lin - Asus
*	email: Hugo_Lin@asus.com
*
*/

#include "usb_firmware_library_headers.h"
#include "bl_flash.h"
#include "OEM_flash.h"
#include <String.h>

#define __disable_irq()     do { EA = 0; } while(0)
#define __enable_irq()      do { EA = 1; } while(0)

#define DATA_FLASH_ADDR     0x7800
__no_init const uint8_t __code data_flash[1024] @ DATA_FLASH_ADDR;

void data_flash_erase(void)
{
    /* Disable interrupt mode so it doesn't fire during FLASH updates */
    __disable_irq();
    
    flashErase(DATA_FLASH_ADDR);
    
    /* Re-enable interrupt mode */
    __enable_irq();
}

void data_flash_read(uint8_t *pdata, uint16_t len)
{
    #if 1
    uint16_t i;
    for (i = 0; i < len; i++) {
        pdata[i] = data_flash[i];
    }
    #else
    memcpy(pdata, data_flash, len);
    #endif
}

void data_flash_program(uint8_t *pdata, uint16_t len)
{  
    /* Disable interrupt mode so it doesn't fire during FLASH updates */
    __disable_irq();
    
    flashWrite((const char *)pdata, len, DATA_FLASH_ADDR);

    /* Re-enable interrupt mode */
    __enable_irq();
}

