/******************************************************************************
   Filename: bl_flash.c

   Copyright 2012 Texas Instruments Incorporated. All rights reserved.

******************************************************************************/


/******************************************************************************
 * INCLUDES
 */
#include <ioCC2544.h>

#include <hal_types.h>
#include <hal_defs.h>
#include "dma.h"

#include "bl_flash.h"


/******************************************************************************
 * Local variables
 */
static DMA_DESC __xdata dmaDescFlash;


/******************************************************************************
 * Local functions
 */

void flashWrite(const char * data, unsigned short length, unsigned short address)
{
    uint16 addr;

    // Set up flash controller with address to write
    // You address 32-bit words through the flash controller
    addr = address >> 2;
    
    FADDRH = HI_UINT16(addr);
    FADDRL = LO_UINT16(addr);

    // Set up DMA for writing to flash
    dmaDescFlash.SRCADDRH   = HI_UINT16(data);
    dmaDescFlash.SRCADDRL   = LO_UINT16(data);
    dmaDescFlash.DESTADDRH  = HI_UINT16(&FWDATA);
    dmaDescFlash.DESTADDRL  = LO_UINT16(&FWDATA);
    dmaDescFlash.LENH       = HI_UINT16(length);
    dmaDescFlash.LENL       = LO_UINT16(length);
    dmaDescFlash.VLEN       = DMA_VLEN_USE_LEN;
    dmaDescFlash.PRIORITY   = DMA_PRI_HIGH;
    dmaDescFlash.M8         = DMA_M8_USE_8_BITS;
    dmaDescFlash.IRQMASK    = DMA_IRQMASK_DISABLE;
    dmaDescFlash.SRCINC     = DMA_SRCINC_1;
    dmaDescFlash.DESTINC    = DMA_DESTINC_0;
    dmaDescFlash.TRIG       = DMA_TRIG_FLASH;
    dmaDescFlash.TMODE      = DMA_TMODE_SINGLE;
    dmaDescFlash.WORDSIZE   = DMA_WORDSIZE_BYTE;

    DMA_SET_ADDR_DESC0(&dmaDescFlash);
    DMA_CHANNEL_ARM(BM(0));
    flashStartWrite();
    while (DMA_CHANNEL_ACTIVE(BM(0)));

    // Set dummy values.
    FADDRH = 0xFF;
    FADDRL = 0xFF;
}


void flashErase(unsigned short address)
{
    uint16 page = HI_UINT16(address >> 2);
    FADDRH = page;
    FADDRL = 0;
    
    flashStartErase();
    
    // Set dummy values.
    FADDRH = 0xFF;
    FADDRL = 0xFF;
}


static __data unsigned char flashPages[32];

unsigned char flashPageIsErased(unsigned short address)
{
    unsigned char page = ((unsigned char*)&address)[1];
    page >>= 2;
    
    return (flashPages[page]);
}

void flashPageMarkErased(unsigned short address)
{
    unsigned char page = ((unsigned char*)&address)[1];
    page >>= 2;
    
    flashPages[page] = 1;
}




