/******************************************************************************
    Filename: flash.h

    Copyright 2012 Texas Instruments, Inc.
******************************************************************************/


#define FLASH_PAGE_SIZE  1024
#define FLASH_PAGES      32


/******************************************************************************
* @fn  flashStartEraseWrite
*
* @brief
*       This function will start flash erase and flash write of the already
*       specified flash page, and can be used as the first flash DMA trigger.
*
* Parameters:
*
* @param  void
*
* @return void
*
******************************************************************************/
void flashStartEraseWrite(void);


/******************************************************************************
* @fn  flashStartErase
*
* @brief
*       This function will start flash erase of the already specified flash page
*       and can be used as the first flash DMA trigger.
*
* Parameters:
*
* @param  void
*
* @return void
*
******************************************************************************/
void flashStartErase(void);


/******************************************************************************
* @fn  flashStartWrite
*
* @brief
*       This function will start flash write (without erasing) of the already
*       specified flash page, and can be used as the first flash DMA trigger.
*
* Parameters:
*
* @param  void
*
* @return void
*
******************************************************************************/
void flashStartWrite(void);


/******************************************************************************
* @fn  flashWrite
*
* @brief
*       This function writes data pointed to by a pSrcAddr to a given address
*       in flash. It is assumed that the location in flash is already erased.
*
* Parameters:
*
* @param  unsigned char * pSrcAddr
*         Pointer to first byte in xdata space which is to be written to flash.
* @param  unsigned short length
*         Number of bytes to write
* @param  unsigned short address
*         Address in flash to write to. Must be aligned on a 4-byte boundary.
*
* @return void
*
******************************************************************************/
void flashWrite(const char * pSrcAddr, unsigned short length, unsigned short address);


/******************************************************************************
* @fn  flashErase
*
* @brief
*       This function erases the content of the flash page touched by the
*       provided address.
*
* Parameters:
*
* @param  unsigned short address
*         Address of the page to erase. Any address within the page can be used.
*
* @return void
*
******************************************************************************/
void flashErase(unsigned short address);


unsigned char flashPageIsErased(unsigned short address);
void flashPageMarkErased(unsigned short address);