/***********************************************************************************

Filename:    simple_hid_dongle.c

Description: Main program of HID dongle.

***********************************************************************************/

/***********************************************************************************
* INCLUDES
*/
#include "usb_hid_cert.h"

__no_init __xdata char bootbyte @ 0x0001;
__no_init __xdata char bootbyte2 @ 0x0002;
__no_init __xdata char bootbyte3 @ 0x0003;
__no_init __xdata char bootbyte4 @ 0x0004;
/***********************************************************************************
* LOCAL VARIABLES
*/
//****************************************************
// Time buffer
uint8 T3_Cnt = 0;

//****************************************************

// wifi chan : 12, 17, 22, 27, 32, 37, 42, 47, 52, 57, 62, 67, 72, 84 //

// pkt var
uint8 pData[MAX_PKT_LEN], PacketLen;

// Factory BER test
uint8_t USB_Reset_Count;
//****************************************************
/* TIMER FUNCTION */
void timer_init(void)
{
    // Timer 3 for Timer
    T3CTL = 0xC6;               // Tick Freqency/64
                                // Start Timer
                                // Overflow interrupt Disable
                                // Clear Counter
                                // Modulo mode
    T3CCTL0 = 0x44;
                                // Interrupt Enable
                                // Compare output mode
                                // Compare mode
                                // No capture
    T3CC0 = 249;                // 249 for 500us period
    T3CCTL1 = 0x00;             // Timer 3 Channel 1 disable
    T3IE = 1;
}

void halTimer3Start(void)
{
    T3CTL |= 0x10;
}

void halTimer3Stop(void)
{
    T3CTL &= ~0x10;
}

/* RF FUNCTION */
void Rf_Proprietary_init(uint8 mode_switch)
{
    // Disable RF
    halRfDisableRadio(FORCE);

    // Enable RF & Setting
    halRfInit(TXPOWER_4_DBM);
    halRfConfig(GFSK_2Mbps_500khz, RX, 32, PKT_CONF_NRF);
    //halRfConfig(GFSK_2Mbps_320khz, RX, 32, PKT_CONF_NRF);

    halRfSetFrequency(2403);
    halRfSetCrc(CRC_16_CCITT);
    halRfSetSyncWord(TEST_SYNCWORD, 0);
    halRfSetRxAddress(0, ADDR_CONF_RX_AUTO_SW1, TEST_ADDRESS, MAX_PKT_LEN);
    halRfEnableRadio();

    // Enable RF interrupt.
    halRfEnableInterrupt(RFIRQF1_TASKDONE);

    // Enable global interrupt.
    halIntOn();
}

/*******************************************************************************
* @fn          main
*
* @brief       Main program
*
* @param       void
*
* @return      int (does not return)
*/
void main(void)
{
    bootbyte = bootbyte; //make compiler happy
    bootbyte2 = bootbyte2; //make compiler happy
    bootbyte3 = bootbyte3; //make compiler happy
    bootbyte4 = bootbyte4; //make compiler happy

    // Enable 32 MHz clock first to calibrate internal 32.7.. KHz rc oscillator.
    CLKCONCMD = 0x80;                 // System Clock Source: 32 Mhz Xtal; Set TICHSPD and CLKSPD to 0 (32 MHz)
    while (CLKCONSTA != 0x80);        // Wait until command has taken effect

#if RF_DEBUG
    // Initialize P0.1 and P0.2 as output for LED's and P0.0|P0.3 as input from buttons
    P0SEL0 = 0x00;                    // Set P0_0 and P0_1 as GPIO , by default
    P0SEL1 = 0x00;                    // Set P0_2 and P0_3 as GPIO , by default
    PDIR   = (BIT0|BIT1|BIT2|BIT3);

    OBSSEL0       = 0xFB;             // set P1.0 to RFC_OBS_CTRL0
    RFC_OBS_CTRL0 = RX_ACTIVE;        // Rx active
    OBSSEL1       = 0xFC;             // set P1.1 to RFC_OBS_CTRL1
    RFC_OBS_CTRL1 = TX_ACTIVE;        // Tx active
#else
    // Initialize P0.1 and P0.2 as output for LED's and P0.0|P0.3 as input from buttons
    P0SEL0 = 0x00;                    // Set P0_0 and P0_1 as GPIO , by default
    P0SEL1 = 0x01;                    // Set P0_2 as UART Tx and P0_3 as GPIO , by default
    PDIR   = (BIT0|BIT1|BIT2);
#endif

    // Setting Timer3 priority Level = 2
    IP1 |= IP1_IPG3;
    IP0 &= ~IP0_IPG3;

    // Setting RF priority Level = 2
    IP1 |= IP1_IPG0;
    IP0 &= ~IP0_IPG0;

    // USB Init
    //usbHidInit();

    // Timer Init
    //timer_init();
    //halTimer3Start();

    // Enable UART
    //oem_uart_init();

    //DBG_dword_MAIN(OEM_EC_Ver);
    //DBG_puts_MAIN("=main service");

    //DBG_byte_MAIN(SLEEPSTA);
    //DBG_puts_MAIN("=reset");

    //check_uart_need_start();

    // Enable Rf
    Rf_Proprietary_init(RX);

    halRfCommand(CMD_RX);

    while(1)
    {
        usbHidProcessEvents();
        OEM_main_service();
    }
}

HAL_ISR_FUNCTION(RF_ISR, RF_VECTOR)
{
    // RF RX Mode
    if (RFIRQF1 & RFIRQF1_TASKDONE) {
        // Check if task ended correctly
        if (PRF.ENDCAUSE == TASK_ENDOK) {
            // Check if packet was received.
            if (RFIRQF1 & RFIRQF1_RXOK) {
                P0_1 ^= 1;
                // Check if packet length is as expected.
                PacketLen = RFD;
                if (PacketLen >= 8) {
                    /* mouse data */
                    pData[0] = RFD;
                    pData[1] = RFD;
                    pData[2] = RFD;
                    pData[3] = RFD;
                    pData[4] = RFD;
                    pData[5] = RFD;
                    pData[6] = RFD;
                    pData[7] = RFD;
                }
            } else if (RFIRQF1 & RFIRQF1_RXIGNORED) {
                P0_3 ^= 1;
                PacketLen = RFD;
            }
        }
        // Set to a undefined value in enum type end cause.
        PRF.ENDCAUSE = TASK_UNDEF;

        // Reset FIFO
        halRfCommand(CMD_FIFO_RESET);

        // start RX
        halRfCommand(CMD_RX);
    }
    // Clear RFIF = 0;
    S1CON   = 0;

    // Clear interrupt.
    RFIRQF1 = 0;
}

HAL_ISR_FUNCTION(T3_ISR, T3_VECTOR)
{
    if (++T3_Cnt >= 2) {
        ServiceFlags.Service_1ms = 1;
        T3_Cnt = 0;
    }

    TIMIF = ~0x07;
    T3IF  = 0;
}
