/***********************************************************************************

  Filename:		usb_hid.h

  Description:	Application support for the HID class

***********************************************************************************/

#ifndef USB_HID_H
#define USB_HID_H

#include "usb_firmware_library_config.h"
#include "usb_hid_reports.h"
#include "usb_class_requests.h"

#define HID_CUSTOM_EPIN_SIZE                0x40
#define HID_CUSTOM_EPOUT_SIZE               0x40

#define HID_OTHER_EPIN_SIZE                 0x11
#define GET_INFO_TIMEOUT                    5   /* hook in 10ms */

enum Report_ID_List {
    HID_CONSUMER_REPORT_ID          = 0x01,
    HID_SYSTEM_REPORT_ID            = 0x02,
    HID_CUSTOM_REPORT_ID            = 0x03,
    HID_KB_REPORT_ID                = 0x04,
};

enum ClassSpecificRequests_List{
    HID_CLASS_BOOT_PORTOCOL         = 0x00,
    HID_CLASS_REPORT_PORTOCOL       = 0x01,
};

enum Prorocol_Command_List{
    CMD_GET_INFO            = 0x12,
    CMD_PROFILE             = 0x50,
    CMD_WRITABLE_DATA       = 0x51,
    CMD_SYNC                = 0x60,

    CMD_TI_DEBUG_WRITE      = 0x7C,
    CMD_TI_DEBUG_READ       = 0xFC,
    CMD_ST_DEBUG_WRITE      = 0x7D,
    CMD_ST_DEBUG_READ       = 0xFD,
    CMD_DG_DEBUG_WRITE      = 0x7E,
    CMD_DG_DEBUG_READ       = 0xFE,

    CMD_ERROR               = 0xFF,

    KEY_GET_DEVICE_INFO     = 0x00,
    KEY_Get_POWER_INFO      = 0x07,
    KEY_SET_POWER           = 0x37,
    KEY_ERROR               = 0xAA,

    RETN_PAIRING_SUCCESS    = 0x01,
    RETN_PAIRING_FAIL       = 0x00,
};

enum R_DeviceInfo_Index_list {
    INDX_GETINFO_DEVINFO    = 0x0000,   //Get Device Info
    INDX_GETINFO_DNGVER     = 0x0001,   //Get Dongle Version
    INDX_GETINFO_DEVEXIST   = 0x0002,   //Get Device Exist
    INDX_GETINFO_DEVSTATUS  = 0x0003,   //Get Device Status
    INDX_GETINFO_CHARGE     = 0x0004,   //Get Charge Info
    INDX_GETINFO_RSOC_ALARM = 0x0005,   //Get Battery_ALARM (Call Back)
};

enum Internal_Command_List{
    ICMD_KEYBOARD           = 0xD0,
    ICMD_SUSPEND            = 0xD1,
    ICMD_RESUME             = 0xD2,
    ICMD_REMOTE_WAKEUP      = 0xD3,
    ICMD_QUERY_STATUS       = 0xD4,
    ICMD_INTERNAL_INFO      = 0xD5,
    ICMD_CONSUMER           = 0xD6,
    ICMD_REPORT_RATE        = 0xD7
};

enum Factory_Protocol_List{
    CMD_FACTORY                     = 0xFA,
    KEY_FCMD_ENABLE                 = 0x00,
        INDEX_FCMD_ENABLE_PASS_WORD     = 0xA5D3,
    KEY_FCMD_SET_LED_COLOR          = 0x01,
    KEY_FCMD_GET_BUTTION_STATUS     = 0x02,
    KEY_FCMD_PERIPHERAL_CHECK       = 0x03,
        INDEX_GET_PERIPHERAL_STATUS     = 0x0000,
        INDEX_GET_CHARGING_STATUS       = 0x0001,
    KEY_FCMD_GET_PROFILE_INFO       = 0x04,
    KEY_RF_PAIRING_SETTING          = 0x05,
        INDEX_CLEAR_PAIRING_DATA        = 0x0000,
        INDEX_GENERATE_PAIRING_DATA     = 0x0001,
        INDEX_READ_PAIRING_DATA         = 0x0002,
        INDEX_WRITE_PAIRING_DATA        = 0x0003,
        INDEX_USB_ENTRY_RF_MODE         = 0x0004,
        INDEX_USB_LEAVE_RF_MODE         = 0x0005,
        INDEX_RF_CONNECT_CHECK          = 0x0006,
    KEY_SET_PROFILE_DEFAULT         = 0x06,
    KEY_SET_SUSPEND_TEST            = 0x07,
    KEY_FCC_TEST                    = 0x08,
        INDEX_FCC_DISABLE               = 0x0000,
        INDEX_FCC_ENABLE                = 0x0001,
    KEY_BER_TEST                    = 0x09,
        INDEX_LEAVE_BER_TEST            = 0x0000,
        INDEX_START_BER_TEST            = 0x0001,
};


//Armoury protocol format:
#define ARMOURY_DATA_LENGTH_MAX 28
typedef struct
{
    uint8_t Command;
    uint8_t Key;
    union {
        uint16_t Int;
        struct {
            uint8_t Two_L;
            uint8_t Three_H;
        };
    } Index;
    uint8_t Data[ARMOURY_DATA_LENGTH_MAX];
} ArmouryDataFormat_t;

typedef union
{
    uint8_t flags;
    struct {
        unsigned rf_data_in       : 1;
        unsigned rf_connect_check : 1;
        unsigned rf_status        : 1;
        unsigned usb_cmd_check    : 1;
        unsigned fcc_test         : 1;
        unsigned ber_test         : 1;
        unsigned reserved         : 2;
    };
} RF_Flag_t;

extern MOUSE_IN_REPORT __data mouse;
extern MOUSE_IN_BOOT_REPORT __data mouse_BootProtocol;
extern Joystick_IN_REPORT __data joystick;
extern uint8_t joystick_data_out;

extern uint8_t USB_OutBuffer[], USB_InBuffer[], RF_StatusBuf[];
extern uint8_t armoury_data_out, armoury_data_out2;

extern uint8_t USB_others_InBuffer[];
extern uint8_t usb_others_data_out;

extern uint8_t ECMD_TimeoutCount;
extern RF_Flag_t rf_flag;
extern uint8_t fcc_duty_len, fcc_hop_freq;

void usbInProcess(void);
void usbOutProcess(void);
void usbHidInit(void);
void usbHidProcessEvents(void);
void usbHidProcessKeyboard(uint8 *pRfData);
void usbHidProcessMouse(uint8 *pRfData);
void Joystick_DataInput(Mouse_ProtocolData_t *Mouse_data, uint8_t *Joystick_key);
void parse_rf_rx_extended_cmd(uint16_t len, uint8_t* pData);
void check_rf_extended_cmd_timeout(void);
void check_rf_connect_timeout(void);
void SetPairing_Status_Return(uint8_t Status);



/*
+------------------------------------------------------------------------------
|  Copyright 2004-2007 Texas Instruments Incorporated. All rights reserved.
|
|  IMPORTANT: Your use of this Software is limited to those specific rights
|  granted under the terms of a software license agreement between the user who
|  downloaded the software, his/her employer (which must be your employer) and
|  Texas Instruments Incorporated (the "License"). You may not use this Software
|  unless you agree to abide by the terms of the License. The License limits
|  your use, and you acknowledge, that the Software may not be modified, copied
|  or distributed unless embedded on a Texas Instruments microcontroller or used
|  solely and exclusively in conjunction with a Texas Instruments radio
|  frequency transceiver, which is integrated into your product. Other than for
|  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
|  works of, modify, distribute, perform, display or sell this Software and/or
|  its documentation for any purpose.
|
|  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
|  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
|  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
|  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
|  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
|  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
|  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING
|  BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
|  CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
|  SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
|  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
|
|  Should you have any questions regarding your right to use this Software,
|  contact Texas Instruments Incorporated at www.TI.com.
|
+------------------------------------------------------------------------------
*/

#endif



