/***********************************************************************************

  Filename:		usb_hid_reports.h

  Description:	Definitions and prototypes for HID reports

***********************************************************************************/

#ifndef USBHIDREPORTS_H
#define USBHIDREPORTS_H

#include "hal_types.h"

typedef struct {
    uint8 ledStatus;
} KEYBOARD_OUT_REPORT;

typedef struct {
    uint8 modifiers;
    uint8 reserved;
    uint8 pKeyCodes[6];
} KEYBOARD_IN_REPORT;

typedef union {
    uint8_t data[8];
    struct {
        struct   {
            int16_t X;
            int16_t Y;

            union {
                uint8_t Byte;
                struct {
                    uint8_t Lift          : 1;
                    uint8_t Right         : 1;
                    uint8_t Middle        : 1;
                    uint8_t Back          : 1;
                    uint8_t Proceed       : 1;
                    uint8_t Reserve       : 3;
                };
            } Key;

            union {
                uint8_t Byte;
                struct {
                    int8_t  Wheel         : 6;
                    uint8_t Internal_CMD  : 1;
                    uint8_t Internal_Info : 1;
                };
            };
        } ReportD;

        struct {
            uint8_t X;
            uint8_t Y;
        } JoystickD;
    };
} Mouse_ProtocolData_t;

typedef union {
    uint8_t data[11];
    struct {
        int16_t X;
        int16_t Y;
        union {
            uint8_t Byte;
            struct{
                uint8_t Lift    :1;
                uint8_t Right   :1;
                uint8_t Middle  :1;
                uint8_t Back    :1;
                uint8_t Proceed :1;
                uint8_t Reserve :3;
            };
        }Key;
        int8_t Wheel;

        uint8_t KeyLatency_NV[5];
    };
} MOUSE_IN_REPORT;

typedef union {
    uint8_t data[4];
    struct {
        int16_t X;
        int16_t Y;
    };
} MOUSE_DATA_REPORT;

typedef union {
    uint8_t data[3];
    struct {
        union {
            uint8_t Byte;
            struct {
                uint8_t Lift      : 1;
                uint8_t Right     : 1;
                uint8_t Reserve   : 6;
            };
        } Key;

        int8_t X;
        int8_t Y;
    };
} MOUSE_IN_BOOT_REPORT;

typedef union {
    uint8_t data[4];
    struct {
        uint8_t X;
        uint8_t Y;
        uint8_t Key;
        uint8_t Hat_Switch;
    };
} Joystick_IN_REPORT;

typedef struct {
    KEYBOARD_OUT_REPORT keyboardOutReport;
    KEYBOARD_IN_REPORT keyboardInReport;
    MOUSE_IN_REPORT mouseInReport;
    uint8 keyboardProtocol;
    uint8 mouseProtocol;
    uint8 customProtocol;
    uint16 keyboardIdleRate;
    uint16 mouseIdleRate;
    uint16 customIdleRate;
} HID_DATA;


extern HID_DATA __data hidData;


uint8 hidSendKeyboardInReport(void);
uint8 hidSendMouseInReport(void);
void hidUpdateKeyboardInReport(KEYBOARD_IN_REPORT __data *pNewReport);
void hidUpdateMouseInReport(MOUSE_IN_REPORT __data *pNewReport);
void hidShowKeyboardLeds(void);

/*
+------------------------------------------------------------------------------
|  Copyright 2004-2007 Texas Instruments Incorporated. All rights reserved.
|
|  IMPORTANT: Your use of this Software is limited to those specific rights
|  granted under the terms of a software license agreement between the user who
|  downloaded the software, his/her employer (which must be your employer) and
|  Texas Instruments Incorporated (the "License"). You may not use this Software
|  unless you agree to abide by the terms of the License. The License limits
|  your use, and you acknowledge, that the Software may not be modified, copied
|  or distributed unless embedded on a Texas Instruments microcontroller or used
|  solely and exclusively in conjunction with a Texas Instruments radio
|  frequency transceiver, which is integrated into your product. Other than for
|  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
|  works of, modify, distribute, perform, display or sell this Software and/or
|  its documentation for any purpose.
|
|  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
|  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
|  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
|  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
|  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
|  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
|  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING
|  BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
|  CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
|  SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
|  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
|
|  Should you have any questions regarding your right to use this Software,
|  contact Texas Instruments Incorporated at www.TI.com.
|
+------------------------------------------------------------------------------
*/

#endif
