
#ifndef OEM_TYPEDEF_H
#define OEM_TYPEDEF_H

//#include "OEM_Include.h"
//#include "OEM_ProjectDef.h"

/* Board Header files */
#include "Board.h"

#define BYTE    uint8_t
#define WORD    uint16_t
#define DWORD   uint32_t
#define FIELD   uint8_t

#ifdef __cplusplus
    #define   __I     volatile             /*!< Defines 'read only' permissions */
#else
    #define   __I     volatile const       /*!< Defines 'read only' permissions */
#endif
#define     __O     volatile             /*!< Defines 'write only' permissions */
#define     __IO    volatile             /*!< Defines 'read / write' permissions */

/* following defines should be used for structure members */
#define     __IM     volatile const      /*! Defines 'read only' structure member permissions */
#define     __OM     volatile            /*! Defines 'write only' structure member permissions */
#define     __IOM    volatile            /*! Defines 'read / write' structure member permissions */

#define BIT0  (1 << 0)
#define BIT1  (1 << 1)
#define BIT2  (1 << 2)
#define BIT3  (1 << 3)
#define BIT4  (1 << 4)
#define BIT5  (1 << 5)
#define BIT6  (1 << 6)
#define BIT7  (1 << 7)
#define BIT8  (1 << 8)
#define BIT9  (1 << 9)
#define BIT10 (1 << 10)
#define BIT11 (1 << 11)
#define BIT12 (1 << 12)
#define BIT13 (1 << 13)
#define BIT14 (1 << 14)
#define BIT15 (1 << 15)
#define BIT16 (1UL << 16)
#define BIT17 (1UL << 17)
#define BIT18 (1UL << 18)
#define BIT19 (1UL << 19)
#define BIT20 (1UL << 20)
#define BIT21 (1UL << 21)
#define BIT22 (1UL << 22)
#define BIT23 (1UL << 23)
#define BIT24 (1UL << 24)
#define BIT25 (1UL << 25)
#define BIT26 (1UL << 26)
#define BIT27 (1UL << 27)
#define BIT28 (1UL << 28)
#define BIT29 (1UL << 29)
#define BIT30 (1UL << 30)
#define BIT31 (1UL << 31)

typedef volatile union _BITS_8 {
    BYTE byte;
    struct {
        FIELD bit0: 1;
        FIELD bit1: 1;
        FIELD bit2: 1;
        FIELD bit3: 1;
        FIELD bit4: 1;
        FIELD bit5: 1;
        FIELD bit6: 1;
        FIELD bit7: 1;
    } fbit;
} BITS_8;

typedef volatile union _BITS_16 {
    WORD word;
    struct {
        FIELD bit0: 1;
        FIELD bit1: 1;
        FIELD bit2: 1;
        FIELD bit3: 1;
        FIELD bit4: 1;
        FIELD bit5: 1;
        FIELD bit6: 1;
        FIELD bit7: 1;
        FIELD bit8: 1;
        FIELD bit9: 1;
        FIELD bit10: 1;
        FIELD bit11: 1;
        FIELD bit12: 1;
        FIELD bit13: 1;
        FIELD bit14: 1;
        FIELD bit15: 1;
    } fbit;
} BITS_16;

typedef volatile union _BITS_32 {
    DWORD dword;
    struct {
        FIELD bit0: 1;
        FIELD bit1: 1;
        FIELD bit2: 1;
        FIELD bit3: 1;
        FIELD bit4: 1;
        FIELD bit5: 1;
        FIELD bit6: 1;
        FIELD bit7: 1;
        FIELD bit8: 1;
        FIELD bit9: 1;
        FIELD bit10: 1;
        FIELD bit11: 1;
        FIELD bit12: 1;
        FIELD bit13: 1;
        FIELD bit14: 1;
        FIELD bit15: 1;
        FIELD bit16: 1;
        FIELD bit17: 1;
        FIELD bit18: 1;
        FIELD bit19: 1;
        FIELD bit20: 1;
        FIELD bit21: 1;
        FIELD bit22: 1;
        FIELD bit23: 1;
        FIELD bit24: 1;
        FIELD bit25: 1;
        FIELD bit26: 1;
        FIELD bit27: 1;
        FIELD bit28: 1;
        FIELD bit29: 1;
        FIELD bit30: 1;
        FIELD bit31: 1;
    } fbit;
} BITS_32;

typedef union {
    uint16_t w;
    uint8_t b[2];
} u16;

typedef union {
    int16_t w;
    int8_t b[2];
} s16;

typedef struct _mouse_surface_material_t {
    uint8_t raw_data_threshold;
    uint8_t min_sq_run;
} mouse_surface_material_t;

typedef union {
    uint8_t data[8];
    struct {
        struct   {
            int16_t X;
            int16_t Y;

            union {
                uint8_t Byte;
                struct {
                    uint8_t Lift            : 1;
                    uint8_t Right           : 1;
                    uint8_t Middle          : 1;
                    uint8_t Back            : 1;
                    uint8_t Proceed         : 1;
                    uint8_t DPI             : 1;
                    uint8_t Pairing         : 1;
                    __IM  uint8_t Reserve   : 1;
                };
            } Key;

            union {
                uint8_t Byte;
                struct {
                    int8_t  Wheel           : 6;
                    uint8_t Internal_CMD    : 1;
                    uint8_t Internal_Info   : 1;
                };
            };
        } ReportD;

        struct {
            uint8_t X;
            uint8_t Y;
        } JoystickD;
    };
} MouseData_t;

typedef union {
    uint8_t data[6];
    struct {
        int16_t X;
        int16_t Y;

        union {
            uint8_t Byte;
            struct {
                uint8_t Lift            : 1;
                uint8_t Right           : 1;
                uint8_t Middle          : 1;
                uint8_t Back            : 1;
                uint8_t Proceed         : 1;
                __IM  uint8_t Reserve   : 3;
            };
        } Key;

        int8_t Wheel;
    };
} Mouse_ReportData_t;

typedef union {
    uint8_t data[0x10];

    struct {
        int8_t Modifiers;
        int8_t Bitmap[0x0F];
    };
} Keyboard_ReportData_t;

typedef union {
    uint8_t data[0x02];

    struct {
        int8_t Consumer;
        int8_t JoystickKey;
    };
} Consumer_ReportData_t;

typedef union {
    uint8_t  byte[12];
    uint16_t word[6];
    struct {
        uint16_t Joystick_X_Min;
        uint16_t Joystick_X_Max;
        uint16_t Joystick_Y_Min;
        uint16_t Joystick_Y_Max;
        uint16_t Joystick_X_Gain;
        uint16_t Joystick_Y_Gain;
    };
} Joystick_CalculateData_t;

// LED Strip structure and enumerate
typedef struct {
    uint8_t R;
    uint8_t G;
    uint8_t B;
} RGB_Color;

typedef struct {
    uint8_t H;
    uint8_t S;
    uint8_t V;
} HSV_Color;
#endif // OEM_TYPEDEF_H

