
#include <ti/drivers/PIN.h>
#include "Board.h"
#include "OEM_Include.h"
#include <ti/sysbios/knl/Clock.h>

#define DPI_ADJUST_FEATURE      1
#define DEDICATE_PAIRING_KEY    1
#define MAKE_KEY_FAST_DEBOUNCE  1

PIN_Handle OEM_PinHandle;
PIN_State OEM_PinState;

uint8_t Key_array[MOUSE_KEY_QUANTITY], Key_last[MOUSE_KEY_QUANTITY];
uint32_t Key_Debounce_Tag[MOUSE_KEY_LATENCY_QUANTITY];
uint32_t Gsync_Key_Debounce;
uint8_t MouseKey = 0;
uint8_t DPIKey = 0;

extern ICall_SyncHandle bleAppEvent;
extern Event_Handle rf_event_handle, rf_event_handle2;

#define ALL_KEY_FREE            ( ((1)<< (GPIO_L_BUTTON)) \
                                | ((1)<< (GPIO_R_BUTTON)) \
                                | ((1)<< (GPIO_MID_BUTTON)) \
                                | ((1)<< (GPIO_S1_BUTTON)) \
                                | ((1)<< (GPIO_S2_BUTTON)) \
                                | ((1)<< (GPIO_DPI_BUTTON)) \
                                | ((1)<< (GPIO_PAIRING_BUTTON)) )
#if RF_DEBUG
const uint8_t Mouse_Key_IO_Table[] = {
    GPIO_PAIRING_BUTTON,
    GPIO_PAIRING_BUTTON,
    GPIO_PAIRING_BUTTON,
    GPIO_PAIRING_BUTTON,
    GPIO_PAIRING_BUTTON,
    GPIO_PAIRING_BUTTON,
    GPIO_PAIRING_BUTTON,
};
#else
const uint8_t Mouse_Key_IO_Table[] = {
    GPIO_L_BUTTON,
    GPIO_R_BUTTON,
    GPIO_MID_BUTTON,
    GPIO_S1_BUTTON,
    GPIO_S2_BUTTON,
    GPIO_DPI_BUTTON,
    GPIO_PAIRING_BUTTON,
};
#endif

void mouse_key_interrupt_enable(void)
{
    PIN_clrPendInterrupt(OEM_PinHandle, GPIO_L_BUTTON);
    PIN_clrPendInterrupt(OEM_PinHandle, GPIO_R_BUTTON);
    PIN_clrPendInterrupt(OEM_PinHandle, GPIO_MID_BUTTON);
    PIN_clrPendInterrupt(OEM_PinHandle, GPIO_S1_BUTTON);
    PIN_clrPendInterrupt(OEM_PinHandle, GPIO_S2_BUTTON);

    PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_L_BUTTON   | PIN_IRQ_NEGEDGE);
    PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_R_BUTTON   | PIN_IRQ_NEGEDGE);
    PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_MID_BUTTON | PIN_IRQ_NEGEDGE);
    PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_S1_BUTTON  | PIN_IRQ_NEGEDGE);
    PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_S2_BUTTON  | PIN_IRQ_NEGEDGE);
}

void mouse_exti_enable(void)
{
    uint8_t qei_phase_a, qei_phase_b;

    qei_phase_a = In_Encoder_QA();
    qei_phase_b = In_Encoder_QB();

    PIN_clrPendInterrupt(OEM_PinHandle, GPIO_DPI_BUTTON);
    PIN_clrPendInterrupt(OEM_PinHandle, GPIO_PAIRING_BUTTON);
    PIN_clrPendInterrupt(OEM_PinHandle, GPIO_ENCODER_QA);
    PIN_clrPendInterrupt(OEM_PinHandle, GPIO_ENCODER_QB);

    if (Is_USB_Mode()) {
        PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_RF_EVENT   | PIN_IRQ_POSEDGE);
    }

    PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_DPI_BUTTON | PIN_IRQ_NEGEDGE);
    PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_PAIRING_BUTTON   | PIN_IRQ_NEGEDGE);
    if (qei_phase_a) {
        PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_ENCODER_QA | PIN_IRQ_NEGEDGE);
    } else {
        PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_ENCODER_QA | PIN_IRQ_POSEDGE);
    }
    if (qei_phase_b) {
        PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_ENCODER_QB | PIN_IRQ_NEGEDGE);
    } else {
        PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_ENCODER_QB | PIN_IRQ_POSEDGE);
    }
}

void mouse_exti_disable(void)
{
    PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_RF_EVENT   | PIN_IRQ_DIS);

    PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_DPI_BUTTON | PIN_IRQ_DIS);
    PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_PAIRING_BUTTON   | PIN_IRQ_DIS);
    PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_ENCODER_QA | PIN_IRQ_DIS);
    PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_ENCODER_QB | PIN_IRQ_DIS);
}

void optic_sensor_exti_enable(void)
{
    PIN_clrPendInterrupt(OEM_PinHandle, GPIO_SS_MOTIONn);

    PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_SS_MOTIONn | PIN_IRQ_NEGEDGE);
}

void optic_sensor_exti_disable(void)
{
    PIN_setConfig(OEM_PinHandle, PIN_BM_IRQ, GPIO_SS_MOTIONn | PIN_IRQ_DIS);
}

void gpioHwiFxn(PIN_Handle hPin, PIN_Id pinId)
{
//    if (system_flag.fbit.sleep_mode) {
//        //Event_post(serviceEvent, OEM_SERVICE_GPIO_WAKEUP_EVENT);
//        mouse_exti_disable();
//        optic_sensor_exti_disable();
//        DBG_puts_Log("GPWakeup");
//    }

    for (int i = 0; i < MOUSE_KEY_LATENCY_QUANTITY; i++) {
        if (Mouse_Key_IO_Table[i] == pinId && !Key_array[i] && !Key_Debounce_Tag[i]) {
            Key_Debounce_Tag[i] = Clock_getTicks();
            DBG_byte_Key(pinId);
            DBG_dword_Key(Key_Debounce_Tag[i]);
            DBG_puts_Key("=BLT1"); /* button latency tag 1*/
        }
    }
}

void OEM_GPIO_Init(void)
{
    OEM_PinHandle = PIN_open(&OEM_PinState, BoardGpioInitTable);
    if (!OEM_PinHandle) {
        DBG_puts_Log("GPIO_Err");
    } else {
        DBG_puts_Log("GPIO_Init");
    }

    for (int i = 0; i < MOUSE_KEY_QUANTITY; i++) {
        Key_last[i] = ((HWREG(GPIO_BASE + GPIO_O_DIN31_0) >> Mouse_Key_IO_Table[i]) & 0x01) ? 0 : 1;
        Key_array[i] = Key_last[i];
    }

    PIN_registerIntCb(OEM_PinHandle, gpioHwiFxn);
}

void OEM_SPI0_Pull_Down(void)
{
    for (int i = 0; i < 3; i++) {
        PIN_add(OEM_PinHandle, OEM_SPI0_Sleep_GPIO_Table[i]);
    }
}

void OEM_SPI_Enter_Sleep_Mode(bool wire_mode)
{
    if (wire_mode) {
        OEM_SPI0_Pull_Down();
    }

    for (int i = 0; i < 3; i++) {
        PIN_add(OEM_PinHandle, OEM_SPI1_Sleep_GPIO_Table[i]);
    }
}

void OEM_SPI_Leave_Sleep_Mode(bool wire_mode)
{
    if (wire_mode) {
        PIN_remove(OEM_PinHandle, MCU_MOSI_SPI0);
        PIN_remove(OEM_PinHandle, MCU_MISO_SPI0);
        PIN_remove(OEM_PinHandle, MCU_SCLK_SPI0);
    }
    PIN_remove(OEM_PinHandle, SS_MOSI_SPI1);
    PIN_remove(OEM_PinHandle, SS_MISO_SPI1);
    PIN_remove(OEM_PinHandle, SS_SCLK_SPI1);
}


