#ifndef __OEM_GPIO__H__
#define __OEM_GPIO__H__

#include "Board.h"

/* write register is around 45~65ns */
#define Out_NCS0_Active()       do{ HWREGB( GPIO_BASE + MCU_NCS_SPI0 ) = 0;} while(0)
#define Out_NCS0_Inactive()     do{ HWREGB( GPIO_BASE + MCU_NCS_SPI0 ) = 1;} while(0)

#define Out_NCS1_Active()       do{ HWREGB( GPIO_BASE + SS_NCS_SPI1 ) = 0;} while(0)
#define Out_NCS1_Inactive()     do{ HWREGB( GPIO_BASE + SS_NCS_SPI1 ) = 1;} while(0)

#define Control_MOSI_To_SDIO()  do{ HWREGB( GPIO_BASE + GPIO_2DIR ) = 1;} while(0)
#define Control_SDIO_To_MISO()  do{ HWREGB( GPIO_BASE + GPIO_2DIR ) = 0;} while(0)

#define Out_SDB_Active()        do{ } while(0)
#define Out_SDB_Inactive()      do{ } while(0)

#define Out_RF_Boot_Active()    do{ HWREGB( GPIO_BASE + GPIO_RF_BOOTn ) = 0;} while(0)
#define Out_RF_Boot_Inactive()  do{ HWREGB( GPIO_BASE + GPIO_RF_BOOTn ) = 1;} while(0)

#define Enable_Power_Latch()    do{ HWREGB( GPIO_BASE + GPIO_CUT_OFF ) = 1;} while(0)
#define Disable_Power_Latch()   do{ HWREGB( GPIO_BASE + GPIO_CUT_OFF ) = 0;} while(0)

#define In_Encoder_QA()         (( HWREG( GPIO_BASE + GPIO_O_DIN31_0 ) >> (GPIO_ENCODER_QA) ) & (0x01))
#define In_Encoder_QB()         (( HWREG( GPIO_BASE + GPIO_O_DIN31_0 ) >> (GPIO_ENCODER_QB) ) & (0x01))
#define QEI_Status()            ( ((1) << (GPIO_ENCODER_QA)) | ((1) << (GPIO_ENCODER_QB)) )
#define In_Encoder_AB()         (( HWREG( GPIO_BASE + GPIO_O_DIN31_0 ) & QEI_Status()) >> (GPIO_ENCODER_QA) )

#define Is_RF_EVENT_Active()    (( HWREG( GPIO_BASE + GPIO_O_DIN31_0 ) >> (GPIO_RF_EVENT) ) & (0x01))

#define Is_WC5V_Active()        ((( HWREG( GPIO_BASE + GPIO_O_DIN31_0 ) >> (GPIO_WC5V_IN ) )  & (0x01)) == 0x01)

#define Is_SS_MOTIONn_Active()  ((( HWREG( GPIO_BASE + GPIO_O_DIN31_0 ) >> (GPIO_SS_MOTIONn) ) & (0x01)) == 0x00)

#define Is_L_Button_Active()    ((( HWREG( GPIO_BASE + GPIO_O_DIN31_0 ) >> (GPIO_L_BUTTON) ) & (0x01)) == 0x00)
#define Is_R_Button_Active()    ((( HWREG( GPIO_BASE + GPIO_O_DIN31_0 ) >> (GPIO_R_BUTTON) ) & (0x01)) == 0x00)

#define Get_CHGOK_Status()      ((( HWREG( GPIO_BASE + GPIO_O_DIN31_0 ) >> (GPIO_CHGOK) ) & (0x01)))
#define Is_CHG_Stop()           ((( HWREG( GPIO_BASE + GPIO_O_DIN31_0 ) >> (GPIO_CHGOK) ) & (0x01)) == 0x01)

#define Wakeup_ST_MCU()         do{ Out_NCS0_Active(); \
                                    sw_delay_1us(10); \
                                    Out_NCS0_Inactive(); \
                                } while(0)

/* call function is around 1us */
//#define Out_NCS0_Active()       do{ PIN_setOutputValue(OEM_PinHandle, MCU_NCS_SPI0, 0);} while(0)
//#define Out_NCS0_Inactive()     do{ PIN_setOutputValue(OEM_PinHandle, MCU_NCS_SPI0, 1);} while(0)

//#define Out_NCS1_Active()       do{ PIN_setOutputValue(OEM_PinHandle, SS_NCS_SPI1, 0);} while(0)
//#define Out_NCS1_Inactive()     do{ PIN_setOutputValue(OEM_PinHandle, SS_NCS_SPI1, 1);} while(0)

//#define IS37L_SDB_ON()          do{ PIN_setOutputValue(OEM_PinHandle, IS37FL_SDB, 1);} while(0)
//#define IS37L_SDB_OFF()         do{ PIN_setOutputValue(OEM_PinHandle, IS37FL_SDB, 0) ;} while(0)

#define MOUSE_KEY_QUANTITY          7
#define MOUSE_KEY_LATENCY_QUANTITY  5

enum Mouse_KeyArray_Index_List {
    KEY_INDEX_L                 = 0x00,
    KEY_INDEX_R                 = 0x01,
    KEY_INDEX_MID               = 0x02,
    KEY_INDEX_S1                = 0x03,
    KEY_INDEX_S2                = 0x04,
    KEY_INDEX_DPI               = 0x05,
    KEY_INDEX_PAIRING           = 0x06,
    KEY_INDEX_S3                = 0x07,
    KEY_INDEX_S4                = 0x08,

    KEY_INDEX_WHEEL_UP          = 0x09,
    KEY_INDEX_WHEEL_DOWN        = 0x0A,

    KEY_INDEX_JOYSTICK_UP       = 0x0B,
    KEY_INDEX_JOYSTICK_DOWN     = 0x0C,
    KEY_INDEX_JOYSTICK_LEFT     = 0x0D,
    KEY_INDEX_JOYSTICK_RIGHT    = 0x0E,

    KEY_INDEX_NUME          = 0xFF,

    KEY_BIT_L       = BIT0,
    KEY_BIT_R       = BIT1,
    KEY_BIT_MID     = BIT2,
    KEY_BIT_S1      = BIT3,
    KEY_BIT_S2      = BIT4,
    KEY_BIT_DPI     = BIT5,
};

extern uint8_t MouseKey;
extern uint32_t Key_Debounce_Tag[MOUSE_KEY_LATENCY_QUANTITY];
extern uint32_t Gsync_Key_Debounce;

void mouse_key_interrupt_enable();
void mouse_exti_enable(void);
void mouse_exti_disable(void);
void optic_sensor_exti_enable(void);
void optic_sensor_exti_disable(void);
void OEM_GPIO_Init(void);
void OEM_SPI0_Pull_Down(void);
void OEM_SPI_Enter_Sleep_Mode(bool wire_mode);
void OEM_SPI_Leave_Sleep_Mode(bool wire_mode);
void oem_key_scan(uint8_t Response_Time);
void Combine_Key_Detect(void);

#endif /*__OEM_GPIO__H__*/
