/***************** INCLUDES ****************/
#include "OEM_Include.h"
#include <icall.h>
#include DEVICE_FAMILY_PATH(driverlib/ioc.h)

#ifdef DEVICE_FAMILY
    #undef DEVICE_FAMILY_PATH
    #define DEVICE_FAMILY_PATH(x) <ti/devices/DEVICE_FAMILY/x>
#else
    #error "You must define DEVICE_FAMILY at the project level as one of cc26x0, cc26x0r2, cc13x0, etc."
#endif

/* Task */
Task_Struct          txTask; /* not static so you can see in ROV */

/* Clock */
static Clock_Struct  RF_Clock;

/* Object or State */
static RF_Object     rfObject;

/* Handle */
RF_Handle            rf_handle;
Event_Handle         rf_event_handle, rf_event_handle2;

/**************** PROTOTYPES ****************/
#if RF_NORMAL_MODE
static void RfTxTaskFunction(UArg arg0, UArg arg1);
#endif

/***************** DEFINES *****************/
#if RF_NORMAL_MODE
static dataQueue_t   dataQueue;

/* Variable */
/* FCC define RF frequency range : 2400 - 2483.5 MHz */
// wifi chan : 12, 17, 22, 27, 32, 37, 42, 47, 52, 57, 62, 67, 72, 84 //
#endif

// pkt var
uint8_t              *packet, *rxAckPacket;
// freq var
uint8_t              *Freq_Table;

#if RF_DEBUG
#define T_IOID0()  { HWREGB(GPIO_BASE + GPIO_O_DOUT3_0 + GPIO_MID_BUTTON) ^= 1; }
#define T_IOID14() { HWREGB(GPIO_BASE + GPIO_O_DOUT3_0 + GPIO_S2_BUTTON) ^= 1; }
#define T_IOID15() { HWREGB(GPIO_BASE + GPIO_O_DOUT3_0 + GPIO_S1_BUTTON) ^= 1; }
#define T_IOID24() { HWREGB(GPIO_BASE + GPIO_O_DOUT3_0 + GPIO_ENCODER_QA) ^= 1; }
#define T_IOID25() { HWREGB(GPIO_BASE + GPIO_O_DOUT3_0 + GPIO_ENCODER_QB) ^= 1; }
#define T_IOID27() { HWREGB(GPIO_BASE + GPIO_O_DOUT3_0 + GPIO_DPI_BUTTON) ^= 1; }
#else
#define T_IOID0()  { }
#define T_IOID14() { }
#define T_IOID15() { }
#define T_IOID24() { }
#define T_IOID25() { }
#define T_IOID27() { }
#endif

/*********** Function Definition ************/
void RfTxTask_Init(void)
{
#if RF_DEBUG
    /* Radio will trigger on the following GPIOs: */
    IOCPortConfigureSet(GPIO_R_BUTTON, IOC_PORT_RFC_GPO0, IOC_IOMODE_NORMAL);  // LNA Enable
    //IOCPortConfigureSet(IOID_12, IOC_PORT_RFC_GPO1, IOC_IOMODE_NORMAL);         // PA  Enable
    //IOCPortConfigureSet(Board_DIO21, IOC_PORT_RFC_GPO2, IOC_IOMODE_NORMAL);     // Synthesizer calibration running
    IOCPortConfigureSet(GPIO_L_BUTTON, IOC_PORT_RFC_GPO3, IOC_IOMODE_NORMAL); // TX  Start
#endif

    uint8 *txTaskStack = ICall_malloc(TX_TASK_STACK_SIZE);
    Task_Params txTaskParams;
    Task_Params_init(&txTaskParams);
    txTaskParams.stackSize = TX_TASK_STACK_SIZE;
    txTaskParams.priority  = TX_TASK_PRIORITY;
    txTaskParams.stack     = txTaskStack;
    txTaskParams.arg0      = (UInt)1000000;

    DBG_puts_MAIN("=NORMAL");
    packet             = ICall_malloc(MAX_PAYLOAD_LEN);
    rxAckPacket        = ICall_malloc(EASYLINK_MAX_DATA_LENGTH);
    Freq_Table         = ICall_malloc(FREQTABLE_LENGTH);

    Task_construct(&txTask, RfTxTaskFunction, &txTaskParams, NULL);
}

/* QUEUE FUNCTION */
void write_queue_to_rf(void)
{
    RF_cmdHIDTx.pktLen = 1;
    RF_cmdHIDTx.retransDelay = 0x600;
    packet[0] = 0;
}

/* CLOCK FUNCTION */
void OEM_UtilClockCallback(UArg arg)
{
    Event_post(rf_event_handle, arg);
}

void OEM_UtilClock_Init(uint8_t ms)
{
    Clock_Handle  Rf_Clk0;

    // Util_Clock Init //
    Rf_Clk0 = Util_constructClock(&RF_Clock, (Clock_FuncPtr)OEM_UtilClockCallback, ms, ms, TRUE, RF_TASK_TIMESLOT_EVENT);
    if (Rf_Clk0 == NULL) {
        DBG_puts_MAIN("Clk_Err");
    } else {
        DBG_puts_MAIN("Clk_Init");
    }
    start_rf_clock();
}

void OEM_Event_Handle_Init(void)
{
    Error_Block   eb;

    Error_init(&eb);

    rf_event_handle = Event_create(NULL, &eb);
    if (rf_event_handle == NULL) {
        //DBG_puts_MAIN("RFCF"); /*rf_event_handle create failed */
    }
}

void start_rf_clock(void)
{
    Util_startClock(&RF_Clock);
}

/* RF FUNCTION */
void OEM_RF_Init(uint8_t sleep_mode)
{
    RF_Params rfParams;

    /* Request access to the radio */
    RF_Params_init(&rfParams);

    // rf parameter setting
    RF_cmdHIDTx.pPkt        = packet;
    RF_cmdHIDTx.pOutput     = &output_tx_Command;
    RF_cmdHIDTx.pRxQueue = &dataQueue;
    RF_cmdFs.condition.rule = COND_ALWAYS;
    RF_cmdFs.pNextOp        = (rfc_radioOp_t*)&RF_cmdHIDTx;

    rf_handle = RF_open(&rfObject, &RF_HID, (RF_RadioSetup*)&cmdHIDRadioSetup, &rfParams);
}

void RF_Ack_Process(rfc_dataEntryGeneral_t *pDataEntry)
{
    uint8_t rxAckLen;
//    uint8_t shift;
//    int diff;

    // rx ack payload
    rxAckLen = pDataEntry->data;
    if (rxAckLen > EASYLINK_MAX_DATA_LENGTH) rxAckLen = EASYLINK_MAX_DATA_LENGTH;

    if (rxAckLen) {
        memcpy(rxAckPacket, &pDataEntry->data + 1, rxAckLen);
    }
}

/* TASK FUNCTION */
static void RfTxTaskFunction(UArg arg0, UArg arg1)
{
    /*
    * Rx buffer includes data entry structure, hdr (len=1byte), dst addr (max of 8 bytes) and data
    * which must be aligned to 4B
    */
    #pragma data_alignment = 4
    static uint8_t rxBuffer[sizeof(rfc_dataEntryGeneral_t) + 1 + EASYLINK_MAX_ADDR_SIZE + EASYLINK_MAX_DATA_LENGTH];

    UInt                    events;
    RF_EventMask            result;
    rfc_dataEntryGeneral_t  *pDataEntry;
    uint8_t                 hop_index = 0;
    bool                    HOP_TRIG = TRUE;

    // Freq Table init
    memset(Freq_Table, 0x00, FREQTABLE_LENGTH);
    Freq_Table[0] = 3;
    Freq_Table[1] = 40;
    Freq_Table[2] = 80;

    RF_cmdHIDTx.syncWord = TEST_SYNCWORD;
    RF_cmdHIDTx.address  = TEST_ADDRESS;

    // clock init
    OEM_UtilClock_Init(2);
    OEM_Event_Handle_Init();

    // pDataEntry init
    pDataEntry                = (rfc_dataEntryGeneral_t*) rxBuffer;
    pDataEntry->length        = EASYLINK_MAX_ADDR_SIZE + EASYLINK_MAX_DATA_LENGTH + 1;
    pDataEntry->config.lenSz  = 1;
    pDataEntry->status        = 0;
    dataQueue.pCurrEntry      = (uint8_t*) pDataEntry;
    dataQueue.pLastEntry      = NULL;

    // RF init
    OEM_RF_Init(0);

    // new packet
    write_queue_to_rf();

    RF_cmdFs.pNextOp = (rfc_radioOp_t*)&RF_cmdHIDTx;
    while(1)
    {
        events = Event_pend(rf_event_handle, Event_Id_NONE, RF_TASK_WAIT_EVENT, BIOS_WAIT_FOREVER);
        if (events & RF_TASK_TIMESLOT_EVENT) {
            // clear dataQueue
            pDataEntry->status     = 0;
            dataQueue.pCurrEntry   = (uint8_t*) pDataEntry;
            dataQueue.pLastEntry   = NULL;

            output_tx_Command.nTx  = 0;
            RF_cmdHIDTx.maxRetrans = 3;

            if (HOP_TRIG) {
                RF_cmdFs.frequency = Freq_Table[hop_index] + 2400;
                result = RF_runCmd(rf_handle, (RF_Op*)&RF_cmdFs, RF_PriorityNormal, NULL, 0);
            } else {
                result = RF_runCmd(rf_handle, (RF_Op*)&RF_cmdHIDTx, RF_PriorityNormal, NULL, 0);
            }

            /*
            * RF_cmdHIDTx.status (received acks):
            * 0x0000     IDLE
            * 0x0001     PENDING
            * 0x0002     ACTIVE
            * 0x5400     HID_DONE_OK
            */
            if (RF_cmdHIDTx.status == HID_DONE_OK) {
                if (HOP_TRIG) HOP_TRIG = FALSE;
            } else if (!HOP_TRIG) {
                // hopping
                T_IOID14();
                HOP_TRIG = TRUE;
                hop_index = 1;
            }
        }

    }
}

