/***************** INCLUDES ****************/
#include <stdio.h>
#include <stdlib.h>
#include <xdc/std.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Types.h>
#include <xdc/runtime/Error.h>

#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Event.h>

/* Drivers */
#include <ti/drivers/rf/RF.h>
#include <ti/drivers/PIN.h>
#include <driverlib/rf_data_entry.h>

/* Board Header files */
#include "Board.h"
#include "smartrf_settings/smartrf_settings.h"
#include "OEM_Service.h"
#include "OEM_uart.h"
#include "OEM_dbg.h"
#include "OEM_Include.h"
#include "util.h"

/***************** DEFINES *****************/
#define RF_DEBUG                  1
#define RF_TASK_TIMESLOT_EVENT    Event_Id_00
#define RF_SUSPEND_EVENT          Event_Id_01
#define RF_RESUME_EVENT           Event_Id_02
#define RF_DETECT_STATUS_EVENT    Event_Id_03
#define RF_REMOTE_WAKEUP_EVENT    Event_Id_04
#define RF_NORMAL_PAIRING_EVENT   Event_Id_05
#define RF_PAIRING_TIMEOUT_EVENT  Event_Id_06
#define RF_TASK_WAIT_EVENT        ((RF_TASK_TIMESLOT_EVENT)  | \
                                   (RF_SUSPEND_EVENT)        | \
                                   (RF_RESUME_EVENT)         | \
                                   (RF_REMOTE_WAKEUP_EVENT)  | \
                                   (RF_NORMAL_PAIRING_EVENT) | \
                                   (RF_PAIRING_TIMEOUT_EVENT)| \
                                   (RF_DETECT_STATUS_EVENT))
#define RF_FCC_TEST_EVENT         Event_Id_00
#define RF_TEST_WAIT_EVENT        (RF_FCC_TEST_EVENT)
#define RF_DUTY_STOP_EVENT        Event_Id_01
#define RF_MOD_DUTY_EVENT         (RF_TASK_TIMESLOT_EVENT | RF_DUTY_STOP_EVENT)
#define TX_TASK_STACK_SIZE        640
#define TX_TASK_PRIORITY          5
#define EASYLINK_MAX_ADDR_SIZE    8
#define EASYLINK_MAX_DATA_LENGTH  32
#define RF_CMD_HANDLE_INVALID     -1
#define RF_isHandleValid(handle)  (handle >= 0)
#define RF_EVENT_MASK             ((RF_EventLastCmdDone)    | \
                                   (RF_EventCmdAborted)     | \
                                   (RF_EventCmdStopped)     | \
                                   (RF_EventCmdCancelled)   | \
                                   (RF_EventCmdPreempted))
#define MAX_QUE_NUM               22
#define HOST_QUE_NUM              2
#define MAX_PAYLOAD_LEN           32
#define FREQTABLE_LENGTH          10
#define BER_FREQ_LENGTH           3
#define BER_PACKETS_NUM           200
#define DEFAULT_ERR_COUNT         2
#define PAIRING_PKT_LEN           FREQTABLE_LENGTH + 6
#define PAIRING_CHECKSUM          4
#define DONGLE_SLEEP_TIMEOUT      10    /* tslot = 2ms, total = 20ms */
#define HOP_SEARCH_TIMEOUT        2500  /* tslot = 2ms, total = 5s   */
#define RF_DISCONNECT_TIMEOUT     200
#define MAX_HOP_CHANNEL           80
#define MIN_HOP_CHANNEL           3
#define RECV_BLOCKING_PASSWORD    0xB32E
#define MOUSE_DATA_SIZE           (sizeof(MouseData_t))

typedef enum {
    NORMAL_MODE = 0x00,
    FIXED_MODE  = 0x01,
    BER_MODE    = 0x02,
    END_MODE    = 0x03,
} BER_TEST_MODE;


/*********** Function Definition ************/
// RF
void RfTxTask_Init(void);
void OEM_RF_Init(uint8_t sleep_mode);
void OEM_RF_DeInit(uint8_t sleep_mode);

// QUEUE
void access_ti_via_rf(uint8_t data_len, uint8_t* pData);
void write_queue_to_rf(void);
int8_t check_rf_queue_size(void);
void RF_Write_to_Armoury(void);

// CLOCK
void start_rf_clock(void);
void stop_rf_clock(void);
