/*
*	OEM_UART.C
*
*   OEM UART related functions.
*
*	Author: Hugo Lin - Asus
*	email: Hugo_Lin@asus.com
*
*/

#include <stdlib.h>
#include <xdc/std.h>

/* Drivers */
#include <ti/drivers/UART.h>
#include <ti/sysbios/knl/Event.h>

/* Board Header files */
#include "Board.h"
#include "OEM_Include.h"

/* Driver handle shared between the task and the callback function */

UART_Handle uart;

#define UART_TX_BUFFER_LENGTH       240
uint8_t uart_tx_buffer[UART_TX_BUFFER_LENGTH] = {0};
uint8_t uart_tx_head, uart_tx_tail, uart_tx_need_check;

void uart_fifo_init( void )
{
#if DBG_MESG
    uart_tx_need_check = 0;
    uart_tx_head = 0;
    uart_tx_tail = 0;
#endif
}

/*
 * Callback function to use the UART in callback mode. It does nothing.
 */
void uartCallback( UART_Handle handle, void *buf, size_t count )
{
#if DBG_MESG
    Event_post( serviceEvent, OEM_SERVICE_UART_EVENT );
#endif
}

void OEM_UART_Init(void)
{
#if DBG_MESG
    UART_init();
    uart_fifo_init();
    UART_Params uartParams;
    UART_Params_init( &uartParams );
    uartParams.writeDataMode = UART_DATA_BINARY;
    uartParams.writeMode = UART_MODE_CALLBACK;
    uartParams.writeCallback = uartCallback;
    uartParams.baudRate = 115200;
    uart = UART_open( Board_UART, &uartParams );

    if (uart == NULL) {
        DBG_puts_Log("UART_Err");
    } else {
        DBG_puts_Log("UART_Init");
    }
#endif
}

void OEM_UART_Sleep_Mode_Init(void)
{
#if DBG_MESG
    UART_Params uartParams;
    UART_Params_init( &uartParams );
    uartParams.writeDataMode = UART_DATA_BINARY;
    uartParams.writeMode = UART_MODE_BLOCKING;
    uartParams.writeCallback = uartCallback;
    uartParams.baudRate = 115200;
    uart = UART_open( Board_UART, &uartParams );

    if (uart == NULL) {
        DBG_puts_Log("SU_Err"); /* sleep UART initialize error */
    }
#endif
}

void OEM_UART_DeInit(void)
{
#if DBG_MESG
    UART_close(uart);
#endif
}

void uart_send_fifo( uint8_t buf )
{
#if DBG_MESG
    int diff;
    diff = ( int )uart_tx_head - ( int )uart_tx_tail;
    if ( diff == -1 || diff >= ( UART_TX_BUFFER_LENGTH - 1 ) ) {
        return; /* data over flow */
    }
    uart_tx_buffer[uart_tx_head] = buf;
    uart_tx_head++;
    if ( uart_tx_head >= UART_TX_BUFFER_LENGTH ) {
        uart_tx_head = 0;
    }
#endif
}

void uart_write_buffer_to_fifo( void )
{
#if DBG_MESG
    int diff;
    if ( uart_tx_head > uart_tx_tail ) {
        diff = uart_tx_head - uart_tx_tail;
    } else {
        diff = UART_TX_BUFFER_LENGTH - uart_tx_tail;
    }
    UART_write( uart, uart_tx_buffer + uart_tx_tail, diff );
    uart_tx_tail += diff;
    if ( uart_tx_tail >= UART_TX_BUFFER_LENGTH ) {
        uart_tx_tail = 0;
    }
#endif

}

void check_uart_need_start( void )
{
#if DBG_MESG
    if ( uart_tx_head != uart_tx_tail ) {
        uart_write_buffer_to_fifo();
    } else {
        uart_tx_need_check = 1;
    }
#endif

}

