/******************************************************************************

 @file       hid_adv_remote.h

 @brief This file contains the HID Advanced Remote Application header file file
        file.

 Group: CMCU, SCS
 Target Device: CC2640R2

 ******************************************************************************

 Copyright (c) 2015-2017, Texas Instruments Incorporated
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 *  Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 *  Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 *  Neither the name of Texas Instruments Incorporated nor the names of
    its contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 ******************************************************************************
 Release Name: simplelink_cc2640r2_sdk_ble_example_pack_01_50_00_62
 Release Date: 2017-11-01 10:38:41
 *****************************************************************************/

#ifndef HIDADVREMOTE_H
#define HIDADVREMOTE_H

#ifdef __cplusplus
extern "C"
{
#endif

/******************************************************************************
 * INCLUDES
 ******************************************************************************/
#include "OEM_Include.h"
/******************************************************************************
 * CONSTANTS
 ******************************************************************************/


// no key ID
#define KEY_NONE                            0x00

// HID keyboard input report length
#define HID_KEYBOARD_IN_RPT_LEN             16

#define BLE_ADV_TIMEOUT                     60
/******************************************************************************
 * MACROS
 ******************************************************************************/
// Delay
#define DELAY_MS(i)      (Task_sleep(((i) * 1000) / Clock_tickPeriod))

//#define CUSTOM_PROFILE_ENABLE
#define SWIFT_PAIR_ENABLE

// Internal Events for RTOS application
#define BLE_ICALL_EVT             ICALL_MSG_EVENT_ID  // Event_Id_31
#define BLE_QUEUE_EVT             UTIL_QUEUE_EVENT_ID // Event_Id_30
#define BLE_HID_MS_REPORT         Event_Id_00
#define BLE_HID_KB_REPORT         Event_Id_01
#define BLE_PAIRING_EVT           Event_Id_02
#define BLE_SLEEP_EVT             Event_Id_03
#define BLE_WAKEUP_EVT            Event_Id_04
#define BLE_CHECK_PARAM_EVT       Event_Id_05
#define BLE_STOP_PAIRING_EVT      Event_Id_06
#define BLE_ALL_EVENTS            ((BLE_ICALL_EVT)      | \
                                  (BLE_QUEUE_EVT)       | \
                                  (BLE_HID_MS_REPORT)   | \
                                  (BLE_HID_KB_REPORT)   | \
                                  (BLE_PAIRING_EVT)     | \
                                  (BLE_SLEEP_EVT)       | \
                                  (BLE_WAKEUP_EVT)      | \
                                  (BLE_CHECK_PARAM_EVT) | \
                                  (BLE_STOP_PAIRING_EVT))

#define BLE_FCC_TEST_EVENT        Event_Id_00
#define BLE_TEST_WAIT_EVENT       (BLE_FCC_TEST_EVENT)

/******************************************************************************
 * FUNCTIONS
 ******************************************************************************/

/******************************************************************************
 * Task creation function for the HID Advanced Remote
 */
void change_advertising_data_without_swift_pair(void);
void HIDAdvRemote_createTask(void);
void RfTxTask_init(void);
void lightclockHandler( UArg arg );
void BLE_Start_Advertising(void);
void access_ti_via_ble(uint16_t len, uint8_t* pData);
void stop_ble_clock(void);
void start_ble_clock(void);
void Set_BLE_IdleTimeOut(uint16_t TimeOut);
/******************************************************************************
 ******************************************************************************/

#ifdef __cplusplus
}
#endif

#endif // HIDADVREMOTE_H
