/******************************************************************************

 @file       main.c

 @brief main entry of the BLE stack sample application.

 Group: CMCU, SCS
 Target Device: CC2640R2

 ******************************************************************************

 Copyright (c) 2013-2017, Texas Instruments Incorporated
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 *  Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 *  Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 *  Neither the name of Texas Instruments Incorporated nor the names of
    its contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 ******************************************************************************
 Release Name: simplelink_cc2640r2_sdk_ble_example_pack_01_50_00_62
 Release Date: 2017-11-01 10:38:41
 *****************************************************************************/

/*******************************************************************************
 * INCLUDES
 */

#include <xdc/runtime/Error.h>

#include <ti/drivers/Power.h>
#include <ti/drivers/power/PowerCC26XX.h>
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Clock.h>
#include <ti/display/Display.h>
#include <ti/sysbios/family/arm/m3/Hwi.h>

#include <icall.h>
#include "hal_assert.h"
#include "Board.h"
#include "bcomdef.h"
#include "peripheral.h"
#include "hiddev.h"
#include "hid_adv_remote.h"

/* Header files required to enable instruction fetch cache */
#include <inc/hw_memmap.h>
#include <driverlib/vims.h>
#include <driverlib/sys_ctrl.h>
#include <inc/hw_fcfg1.h>

#ifndef USE_DEFAULT_USER_CFG
    #include "ble_user_config.h"
    #include "OEM_Include.h"

    // BLE user defined configuration
    #ifdef ICALL_JT
        icall_userCfg_t user0Cfg = BLE_USER_CFG;
    #else  /* ! ICALL_JT */
        bleUserCfg_t user0Cfg = BLE_USER_CFG;
    #endif /* ICALL_JT */

#endif // USE_DEFAULT_USER_CFG

/*******************************************************************************
 * MACROS
 */

/*******************************************************************************
 * CONSTANTS
 */

/*******************************************************************************
 * TYPEDEFS
 */

/*******************************************************************************
 * LOCAL VARIABLES
 */

/*******************************************************************************
 * GLOBAL VARIABLES
 */
void execHandlerHook(Hwi_ExcContext *ctx)
{
    //delay(500); //about 2.5ms
    DBG_puts_Log("HF!"); /* hardfault !!! */
    check_uart_need_start();
    //delay(500); //about 2.5ms
#ifndef OEM_TEST_MODE
    HAL_SYSTEM_RESET();
#endif
    while(1);
}

/*******************************************************************************
 * EXTERNS
 */
extern void AssertHandler( uint8 assertCause, uint8 assertSubcause );

/*******************************************************************************
 * @fn          Main
 *
 * @brief       Application Main
 *
 * input parameters
 *
 * @param       None.
 *
 * output parameters
 *
 * @param       None.
 *
 * @return      None.
 */
int main()
{
    /* Register Application callback to trap asserts raised in the Stack */
    RegisterAssertCback(AssertHandler);
    PIN_init(BoardGpioInitTable);

#if DBG_MESG
    OEM_UART_Init();
#endif
    OEM_GPIO_Init();
    //OEM_TRNG_Init();

#if 1
#ifdef ICALL_JT
    /* Update User Configuration of the stack */
    user0Cfg.appServiceInfo->timerTickPeriod = Clock_tickPeriod;
    user0Cfg.appServiceInfo->timerMaxMillisecond  = ICall_getMaxMSecs();
#endif  /* ICALL_JT */
    /* Initialize ICall module */
    ICall_init();
#endif

    osal_snv_init();

    DBG_dword_MAIN(OEM_EC_Ver);
    Set_RF_Mode();

    /* RF mode */
    DBG_puts_MAIN("=RF");
    RfTxTask_Init();
    MainService_createTask();
    oem_watchdog_init();

#ifndef POWER_SAVING
    /* Set constraints for Standby, powerdown and idle mode */
    Power_setConstraint(PowerCC26XX_SB_DISALLOW);
    Power_setConstraint(PowerCC26XX_IDLE_PD_DISALLOW);
#endif // POWER_SAVING

    /* enable interrupts and start SYS/BIOS */
    BIOS_start();
    return 0;
}

/*******************************************************************************
 * @fn          AssertHandler
 *
 * @brief       This is the Application's callback handler for asserts raised
 *              in the stack.  When EXT_HAL_ASSERT is defined in the Stack
 *              project this function will be called when an assert is raised,
 *              and can be used to observe or trap a violation from expected
 *              behavior.
 *
 *              As an example, for Heap allocation failures the Stack will raise
 *              HAL_ASSERT_CAUSE_OUT_OF_MEMORY as the assertCause and
 *              HAL_ASSERT_SUBCAUSE_NONE as the assertSubcause.  An application
 *              developer could trap any malloc failure on the stack by calling
 *              HAL_ASSERT_SPINLOCK under the matching case.
 *
 *              An application developer is encouraged to extend this function
 *              for use by their own application.  To do this, add hal_assert.c
 *              to your project workspace, the path to hal_assert.h (this can
 *              be found on the stack side). Asserts are raised by including
 *              hal_assert.h and using macro HAL_ASSERT(cause) to raise an
 *              assert with argument assertCause.  the assertSubcause may be
 *              optionally set by macro HAL_ASSERT_SET_SUBCAUSE(subCause) prior
 *              to asserting the cause it describes. More information is
 *              available in hal_assert.h.
 *
 * input parameters
 *
 * @param       assertCause    - Assert cause as defined in hal_assert.h.
 * @param       assertSubcause - Optional assert subcause (see hal_assert.h).
 *
 * output parameters
 *
 * @param       None.
 *
 * @return      None.
 */
void AssertHandler( uint8 assertCause, uint8 assertSubcause )
{
    // check the assert cause
    switch ( assertCause ) {
#ifdef EXT_HAL_ASSERT
    case HAL_ASSERT_CAUSE_FALSE:
        while(1);
        break;
    case HAL_ASSERT_CAUSE_TRUE:
        while(1);
        break;
    case HAL_ASSERT_CAUSE_INTERNAL_ERROR:
        while(1);
        break;
    case HAL_ASSERT_CAUSE_HW_ERROR:
        while(1);
        break;
    case HAL_ASSERT_CAUSE_OUT_OF_MEMORY:
        while(1);
        break;
    case HAL_ASSERT_CAUSE_ICALL_ABORT:
        while(1);
        break;
    case HAL_ASSERT_CAUSE_ICALL_TIMEOUT:
        while(1);
        break;
    case HAL_ASSERT_CAUSE_WRONG_API_CALL:
        while(1);
        break;
    case HAL_ASSERT_CAUSE_HARDWARE_ERROR:
        while(1);
        break;
    case HAL_ASSERT_CAUSE_RF_DRIVER_ERROR:
        while(1);
        break;
#endif
    default:
        HAL_ASSERT_SPINLOCK;
    }

    return;
}


/*******************************************************************************
 * @fn          smallErrorHook
 *
 * @brief       Error handler to be hooked into TI-RTOS.
 *
 * input parameters
 *
 * @param       eb - Pointer to Error Block.
 *
 * output parameters
 *
 * @param       None.
 *
 * @return      None.
 */
void smallErrorHook( Error_Block *eb )
{
    for ( ;; );
}


/*******************************************************************************
 */
