/******************************************************************************
*  Filename:       hid_cmd.h
*  Revised:        $Date$
*  Revision:       $Revision$
*
*  Description:    CC26xxR2 API for HID commands
*
*  Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
*
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*    Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
******************************************************************************/

#ifndef __RF_HID_CMD_H
#define __RF_HID_CMD_H

#ifndef __RFC_STRUCT
#ifdef __GNUC__
#define __RFC_STRUCT __attribute__ ((aligned (4)))
#else
#define __RFC_STRUCT
#endif
#endif

//! \addtogroup rfc
//! @{

//! \addtogroup hid_cmd
//! @{

#ifdef DEVICE_FAMILY
    #undef DEVICE_FAMILY_PATH
    #define DEVICE_FAMILY_PATH(x) <ti/devices/DEVICE_FAMILY/x>
#else
    #error "You must define DEVICE_FAMILY at the project level as one of cc26x0, cc26x0r2, cc13x0, etc."
#endif

#include DEVICE_FAMILY_PATH(driverlib/rf_mailbox.h)
#include DEVICE_FAMILY_PATH(driverlib/rf_common_cmd.h)
#include <stdint.h>


typedef struct __RFC_STRUCT rfc_CMD_HID_TX_s rfc_CMD_HID_TX_t;
typedef struct __RFC_STRUCT rfc_hidRxTxOutput_s rfc_hidRxTxOutput_t;
typedef struct __RFC_STRUCT rfc_hidRxStatus_s rfc_hidRxStatus_t;

//! \addtogroup CMD_HID_TX
//! @{
#define CMD_HID_TX                                              0x5801
//! HID Transmit Command with Auto Retransmission
struct __RFC_STRUCT rfc_CMD_HID_TX_s {
   uint16_t commandNo;                  //!<        The command ID number 0x5801
   uint16_t status;                     //!< \brief An integer telling the status of the command. This value is
                                        //!<        updated by the radio CPU during operation and may be read by the
                                        //!<        system CPU at any time.
   rfc_radioOp_t *pNextOp;              //!<        Pointer to the next operation to run after this operation is done
   ratmr_t startTime;                   //!<        Absolute or relative start time (depending on the value of <code>startTrigger</code>)
   struct {
      uint8_t triggerType:4;            //!<        The type of trigger
      uint8_t bEnaCmd:1;                //!< \brief 0: No alternative trigger command<br>
                                        //!<        1: CMD_TRIGGER can be used as an alternative trigger
      uint8_t triggerNo:2;              //!<        The trigger number of the CMD_TRIGGER command that triggers this action
      uint8_t pastTrig:1;               //!< \brief 0: A trigger in the past is never triggered, or for start of commands, give an error<br>
                                        //!<        1: A trigger in the past is triggered as soon as possible
   } startTrigger;                      //!<        Identification of the trigger that starts the operation
   struct {
      uint8_t rule:4;                   //!<        Condition for running next command: Rule for how to proceed
      uint8_t nSkip:4;                  //!<        Number of skips + 1 if the rule involves skipping. 0: same, 1: next, 2: skip next, ...
   } condition;
   struct {
      uint8_t bFsOff:1;                 //!< \brief 0: Keep frequency synth on after command<br>
                                        //!<        1: Turn frequency synth off after command
      uint8_t bAutoRetransmit:1;        //!< \brief 0: Do not listen for ACK<br>
                                        //!<        1: Listen for ACK and retransmit if missing
      uint8_t bVarLen:1;                //!< \brief 0: Fixed length mode<br>
                                        //!<        1: Variable length mode
      uint8_t hdrMode:1;                //!< \brief 0: 9-bit header<br>
                                        //!<        1: 10-bit header
      uint8_t bIncludeAddr:1;           //!< \brief 0: Include address byte after sync word<br>
                                        //!<        1: Do not include address byte after sync word
      uint8_t hdrConf:2;                //!< \brief 0: Automatically generate header (no header byte in buffer\)<br>
                                        //!<        1: Insert NO_ACK field from TX buffer<br>
                                        //!<        2: Insert SEQ field from TX buffer<br>
                                        //!<        3: Insert SEQ and NO_ACK field from TX buffer
      uint8_t bFixedTxLen:1;            //!< \brief 0: Use actual length in header when sending packet<br>
                                        //!<        1: Use fixed word in length field of header when sending packet
                                        //!<        (only for peer without variable length packets)
   } pktConf;
   struct {
      uint8_t bAutoFlushIgnored:1;      //!<        If 1, automatically remove empty ACKs from RX queue
      uint8_t bAutoFlushCrcErr:1;       //!<        If 1, automatically remove packets with CRC error from Rx queue
      uint8_t bIncludeAddr:1;           //!<        If 1, the received address byte is included in the Rx queue
      uint8_t bIncludeHdr:1;            //!<        If 1, the received header is included in the Rx queue
      uint8_t bIncludeCrc:1;            //!<        If 1, include the received CRC field in the stored packet; otherwise discard it
      uint8_t bAppendRssi:1;            //!<        If 1, append an RSSI byte to the packet in the Rx queue
      uint8_t bAppendStatus:1;          //!<        If 1, append a status byte to the packet in the Rx queue
      uint8_t bAppendTimestamp:1;       //!<        If 1, append a timestamp to the packet in the Rx queue
   } rxConf;                            //!<        Receive entry configuration
   uint32_t syncWord;                   //!<        Sync word to send
   uint8_t address;                     //!<        Address byte
   uint8_t seqNo;                       //!<        Sequence number to use for next packet
   uint8_t maxAckLen;                   //!<        Maximum length of ACKs
   uint8_t pktLen;                      //!<        Length of transmitted packet
   uint8_t maxRetrans;                  //!<        Maximum number of retransmissions
   uint8_t __dummy0;
   uint16_t retransDelay;               //!<        Number of RAT ticks from start of transmission of a packet to retransmission
   uint8_t* pPkt;                       //!<        Pointer to packet
   dataQueue_t* pRxQueue;               //!<        Pointer to receive queue for ACKs
   rfc_hidRxTxOutput_t *pOutput;        //!<        Pointer to output structure
};

//! @}

//! \addtogroup hidRxTxOutput
//! @{
//! Output structure for CMD_HID_TX

struct __RFC_STRUCT rfc_hidRxTxOutput_s {
   uint8_t nTx;                         //!<        Number of packets or acknowledgements transmitted
   uint8_t nRxOk;                       //!<        Number of packets that have been received with CRC OK
   uint8_t nRxNok;                      //!<        Number of packets that have been received with CRC error
   uint8_t nRxIgnored;                  //!<        Number of packets ignored as retransmissions or empty ACKs
   uint8_t nRxBufFull;                  //!<        Number of packets that have been received and discarded due to lack of buffer space
   uint8_t __dummy0;
   uint8_t __dummy1;
   int8_t lastRssi;                     //!<        RSSI of last received packet
   ratmr_t timeStamp;                   //!<        Time stamp of last received packet
};

//! @}

//! \addtogroup hidRxStatus
//! @{
//! Receive status byte that may be appended to message in receive buffer

struct __RFC_STRUCT rfc_hidRxStatus_s {
   struct {
      uint8_t :6;
      uint8_t bIgnore:1;                //!<        1 if the packet is marked as ignored, 0 otherwise
      uint8_t bCrcErr:1;                //!<        1 if the packet was received with CRC error, 0 otherwise
   } status;
};

//! @}

//! @}
//! @}
#endif
