//------------------------------------------------------------------------------
// TI Confidential - NDA Restrictions
//
// Copyright (c) 2013 Texas Instruments, Inc.
//
//    This is an unpublished work created in the year stated above.
//    Texas Instruments owns all rights in and to this work and
//    intends to maintain and protect it as an unpublished copyright.
//    In the event of either inadvertent or deliberate publication,
//    the above stated date shall be treated as the year of first
//    publication. In the event of such publication, Texas Instruments
//    intends to enforce its rights in the work under the copyright
//    laws as a published work.
//
//------------------------------------------------------------------------------
///
/// \file            hid_mailbox.h
/// \brief           Definitions for HID mode radio interface
///
/// \author          Low Power RF Wireless Business Unit
///                  Helge Coward (h.coward@ti.com)
///
/// \date            Wed Dec  4 16:07:09 CET 2013
///
//-----------------------------------------------------------------------------

#ifndef _HID_MAILBOX_H
#define _HID_MAILBOX_H



/// \name Radio operation status
/// Radio operation status format:
/// Bits 15:12: Protocol
///             0101: HID
/// Bits 11:10: Type
///             00: Not finished
///             01: Done successfully
///             10: Done with error
/// Bit  9:     N/A
/// Bits 8:6:   Sub-type:
///             000: HID
/// Bits 5:0:   Identifier

/// \name Operation finished normally
///@{
#define HID_DONE_OK            0x5400  ///< Operation ended normally
#define HID_DONE_RXTIMEOUT     0x5401  ///< Operation stopped after end trigger while waiting for sync
#define HID_DONE_NOSYNC        0x5402  ///< Timeout of subsequent Rx
#define HID_DONE_RXERR         0x5403  ///< Operation ended after CRC error
#define HID_DONE_ENDED         0x5404  ///< Operation stopped after end trigger during reception
#define HID_DONE_STOPPED       0x5405  ///< Operation stopped after stop command
#define HID_DONE_ABORT         0x5406  ///< Operation aborted by abort command
///@}
/// \name Operation finished with error
///@{
#define HID_ERROR_PAR          0x5800  ///< Illegal parameter
#define HID_ERROR_RXBUF        0x5801  ///< No available Rx buffer at the start of a packet
#define HID_ERROR_NO_SETUP     0x5802  ///< Radio was not set up in a compatible mode
#define HID_ERROR_NO_FS        0x5803  ///< Synth was not programmed when running Rx or Tx
#define HID_ERROR_RXOVF        0x5804  ///< Rx overflow observed during operation
#define HID_ERROR_TXUNF        0x5805  ///< Tx underflow observed during operation
///@}
///@}

#endif
