/******************************************************************************

 @file       bim_main.c

 @brief This module contains the definitions for the main functionality of a
        Boot  Image Manager.

 Group: CMCU, SCS
 Target Device: CC2640R2

 ******************************************************************************
 
 Copyright (c) 2012-2017, Texas Instruments Incorporated
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 *  Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 *  Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 *  Neither the name of Texas Instruments Incorporated nor the names of
    its contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 ******************************************************************************
 Release Name: simplelink_cc2640r2_sdk_01_50_00_58
 Release Date: 2017-10-17 18:09:51
 *****************************************************************************/

/*******************************************************************************
 *                                          Includes
 */

#include <inc/hw_device.h>
#include <driverlib/sys_ctrl.h>
#include <driverlib/aon_event.h>
#include <driverlib/osc.h>
#include <driverlib/setup.h>
#include <driverlib/pwr_ctrl.h>

#include "hal_flash.h"
#include "hal_types.h"
#include "oad_image_header.h"
#include "gpio_debug.h"
#include "bim_util.h"

#define OEM_USER_CODE_SIZE                  (0x1B000)
#define OEM_APPLICATION_ADDRESS             (0x00000000)

int AP_Checksum(uint32_t AP_Size)
{
    int i;
    uint32_t checksum = 0;
    uint32_t* rom_value;
    rom_value = (uint32_t*) (OEM_APPLICATION_ADDRESS);
    for (i= 0; i < AP_Size/4 -1; i++)
    {
        checksum += *rom_value;
        rom_value++;
    }
    
    //DBG_dword_MAIN(checksum);
    if (checksum != *rom_value)
    {
        //DBG_puts_MAIN("=CKF"); /* checksum fail */
        return 1;
    }
    else
    {
        //DBG_puts_MAIN("=CKP"); /* checksum pass */
        return 0;
    }
}
/*******************************************************************************
 * @fn          main
 *
 * @brief       C-code main function.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 */
void main(void)
{
   if (!AP_Checksum(OEM_USER_CODE_SIZE)) {
    jumpToPrgEntry(OEM_APPLICATION_ADDRESS);
  }
  
  powerUpGpio();
  
  while(1) {
    /* puts uart Tx signal to notify bootloader mode */
    blinkLed();
    delay(10000);
  }
}

/**************************************************************************************************
*/
